;;; rascon.el --- Remote Access Service Contorol package for GNU Emacs.

;;; Copyright (C) 1998, 1999 Tatsuya Ichikawa
;;; Author: Tatsuya Ichikawa <t-ichi@po.shiojiri.ne.jp>
;;;         Hideyuki SHIRAI <Shirai@rdmg.mgcs.mei.co.jp>
;;;         Shuichi Kitaguchi <kit@Mew.org>
;;; Maintainer: Tatsuya Ichikawa <t-ichi@po.shiojiri.ne.jp>
;;;             Hideyuki SHIRAI <Shirai@rdmg.mgcs.mei.co.jp>
;;;             Shuichi Kitaguchi <kit@Mew.org>
;;; Version: 0.22
;;; Created:  Fri Jun 19 11:05:38 1998
;;; Revised:  Wed Jul 21 22:07:00 1999
;;; Keywords: Dialup , Hangup
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.

;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to the
;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;;; Boston, MA 02111-1307, USA.
;;;
;;; Commentary:
;;;
;;; How to use.
;;;
;;;    1. Install
;;;
;;;       Add (require 'rascon) to your .emacs.
;;;
;;;    2. Generate RASCON entry table ... M-x rascon-get-entry
;;;    3. Set current configuration   ... M-x rascon-set-current-entry
;;;    4. Dialup                      ... M-x rascon-dialup
;;;    5. Hangup                      ... M-x rascon-hangup
;;;    6. Dialup process kill         ... M-x rascon-kill
;;;    7. Show current entry's status ... M-x rascon-show-current-status
;;;


;;; Code:

(defconst rascon-appname "RASCON")
(defconst rascon-version "0.22")

(require 'cl)

(unless (and (condition-case ()
		 (require 'custom)
	       (file-error nil))
	     (fboundp 'defgroup)
	     (fboundp 'defcustom))
  (require 'backquote)
  (defmacro defgroup (&rest args))
  (defmacro defcustom (symbol value &optional doc &rest args)
    (` (defvar (, symbol) (, value) (, doc))))
  )

;; Group variable.
(defgroup rascon nil
  "Remote Access Service control package."
  :prefix "rascon-"
  :group 'application)

(defcustom rascon-use-dialkeeper
  (string-match "-windows9" system-configuration)
  "*If t , rascon use Dial-Keeper."
  :group 'rascon
  :type 'boolean)

(defcustom rascon-control-program
  "RasCon.exe"
  "*Rascon external program."
  :group 'rascon
  :type 'string)

(defcustom rascon-dialup-complete-hook nil
  "*Hook called at dialup complete success."
  :group 'rascon
  :type 'hooks)

(defcustom rascon-dialup-failure-hook nil
  "*Hook called at dialup failure."
  :group 'rascon
  :type 'hooks)

(defcustom rascon-redial-time 15
  "*Re-dial interval time(sec)."
  :group 'rascon
  :type 'integer)

(defcustom rascon-redial-num 10
  "*Number of Re-dial."
  :group 'rascon
  :type 'integer)

(defcustom rascon-use-menu t
  "*If t , rascon enable to control from menubar."
  :group 'rascon
  :type 'boolean)

(defcustom rascon-use-menu-ja nil
  "*If t , rascon enable to control from menubar in Japanese."
  :group 'rascon
  :type 'boolean)

(defcustom rascon-use-mode-line t
  "*If t , display a connected entry at mode-line."
  :group 'rascon
  :type 'boolean)

(defcustom rascon-frame-title-function
  '(lambda (indicator)
     (format "Meadow%s" indicator))
  "*Display function for a connected entry at frame title."
  :group 'rascon
  :type 'function)

;;; Internal constants.
(defconst rascon-buffer            " *RASCON*")
;;;(defconst rascon-proc-buffer       " *RASCON*PROC*")

;;; Internal variables.
(defvar rascon-entry-list        nil
  "*Dialup entry list.")
(defvar rascon-current-entry     nil
  "*Current entry for Dialup/Hangup.")
(defvar rascon-proc              nil)
(defvar rascon-dialup-complete   nil)
(defvar rascon-saved-line        nil)
(defvar rascon-mode-indicator    nil)
(defvar rascon-current-passwd    nil)
(defvar rascon-debug             nil)

(defvar rascon-point-status      nil)
(defvar rascon-point-num         nil)
(defvar rascon-point-name        nil)

(eval-when-compile 
  (require 'comint))

;;;
;;; menu set.
(if rascon-use-menu
    (progn
      (require 'easymenu)
      (define-key-after
	(lookup-key global-map [menu-bar])
	[RasCon]
	(cons "RasCon" (make-sparse-keymap "RasCon"))
	'help)
      (easy-menu-define
       rascon-menu
       global-map
       "RasCon"
       (if rascon-use-menu-ja
	   '("RasCon"
	     ["$B@\B3(B"              rascon-dialup (not rascon-mode-indicator)]
	     ["$B@\B3@h@_Dj(B"        rascon-set-current-entry t]
	     ["$B8=:_$N>uBVI=<((B"    rascon-show-current-status t]
	     "----"
	     ["$B@ZCG(B"              rascon-hangup rascon-mode-indicator]
	     "----"
	     ["$B@\B3@h0lMw$r<hF@(B"  rascon-get-entry t]
	     "----"
	     ["$B@\B3Cf;_(B"          rascon-kill (processp rascon-proc)])
	 '("RasCon"
	   ["Dialup"              rascon-dialup (not rascon-mode-indicator)]
	   ["Set Current Entry"   rascon-set-current-entry t]
	   ["Show Current Status" rascon-show-current-status t]
	   "----"
	   ["Hangup"              rascon-hangup rascon-mode-indicator]
	   "----"
	   ["Get Dialup Entry"    rascon-get-entry t]
	   "----"
	   ["Dialup Process Kill" rascon-kill (processp rascon-proc)]))
       )))

;;;
;;; disp. mode-line
(if (and rascon-use-mode-line
	 (not (member 'rascon-mode-indicator mode-line-format)))
    (progn
      (delete "-%-" mode-line-format)
      (setq-default mode-line-format 
		    (append mode-line-format 
			    (list
			     "--"
			     'rascon-mode-indicator
			     "-%-")))))

;;;
;;; disp. frame title
(defun rascon-title-line-updater ()
  (if (not (functionp rascon-frame-title-function))
      nil
    (modify-frame-parameters
     nil
     (list
      (cons 'name
	    (concat (funcall rascon-frame-title-function
			     (if rascon-mode-indicator
				 rascon-mode-indicator
			       ""))
		    ":" (buffer-name)))))))

(add-hook 'post-command-hook 'rascon-title-line-updater)

;;
;; Generate rascon entry-list from RasCon.exe
(defun rascon-generate-entry-table ()
  "*Execute rascon.exe -l and get entry table from *RASCON* buffer."
  (if (not (get-buffer rascon-buffer))
      (get-buffer-create rascon-buffer))
  (set-buffer rascon-buffer)
  (erase-buffer)
  (setq rascon-entry-list nil)
  (modify-coding-system-alist 
   'process rascon-control-program
   (cons file-name-coding-system file-name-coding-system))
  (call-process rascon-control-program
		nil
		rascon-buffer
		nil
		"-l")
  (goto-char (point-min))
  (let ((connect-entry nil)
	beg end)
    (while (not (eobp))
      (if (re-search-forward "[0-9] : " nil t)
	  (progn
	    (setq end (- (match-end 0) 3))
	    (setq rascon-point-num (buffer-substring
				    (match-beginning 0)
				    end))))
      (if (re-search-forward ".* : " nil t)
	  (progn
	    (setq end (- (match-end 0) 3))
	    (setq rascon-point-status (buffer-substring
				       (match-beginning 0)
				       end))
	    (goto-char (- (match-end 0) 3))))
      (if (re-search-forward ": .*$" nil t)
	  (progn
	    (setq beg (+ (match-beginning 0) 2))
	    (setq rascon-point-name (buffer-substring
				     beg
				     (match-end 0)))))
      (if (and (stringp rascon-point-num)
	       (stringp rascon-point-status)
	       (stringp rascon-point-name))
	  (progn
	    (setq rascon-entry-list
		(append rascon-entry-list
			(list
			 (list
			  rascon-point-name
			  rascon-point-num
			  rascon-point-status))))
	  (if (string-match "^connect " rascon-point-status)
	      (progn
		(setq connect-entry t)
		(setq rascon-current-entry
		      (list
		       rascon-point-name
		       rascon-point-num
		       rascon-point-status))
		(setq rascon-mode-indicator
		      (concat "[ " rascon-point-name " ]"))))))
      (forward-line))
    (if connect-entry
	()
      (setq rascon-mode-indicator nil)
      (if rascon-current-entry
	  (setq rascon-current-entry
		(list
		 (car rascon-current-entry)
		 (nth 1 rascon-current-entry)
		 rascon-point-status)))))
  ;;
  (if (not rascon-entry-list) nil t))

;;
;; Generate rascon entry.
(defun rascon-get-entry ()
  "*Generate dialup entry table."
  (interactive)
  (if (processp rascon-proc)
      (message "rascon is running. Try again later.")
    (let ((dspmsg (interactive-p))
	  (gentbl (rascon-generate-entry-table)))
      (if (not gentbl)
	  (and dspmsg
	       (message "Getting RASCON entry .... Failed."))
	(and dspmsg
	     (message "Getting RASCON entry .... Done."))))))

;;
;; Show current entry's status.
(defun rascon-show-current-status ()
  "*Show current entry's status."
  (interactive)
  (if (processp rascon-proc)
      (message "rascon is running. Try again later.")
    (if (not rascon-current-entry)
	(rascon-set-current-entry))
    (message
     (format "Status : %s is now %s."
	     (car rascon-current-entry)
	     (progn
	       (string-match "\\([^ ]*\\)" (nth 2 rascon-current-entry))
	       (substring (nth 2 rascon-current-entry) 0 (match-end 1)))))))

;;
;; Set current dialup entry.
(defun rascon-set-current-entry (&optional current)
  "*Set current dialup entry."
  (interactive)
  (if (not rascon-entry-list)
      (rascon-get-entry))
  (let ((name
	 (if (and
	      current
	      (memq t
		    (mapcar
		     (lambda (x)
		       (if (string-match (car x) current) t nil))
		     rascon-entry-list)))
	     current
	   (completing-read "Select dialup entry: "
			    rascon-entry-list
			    nil t nil))))
    (mapcar
     (lambda (x)
       (if (string-equal name (car x))
	   (setq rascon-current-entry x)))
     rascon-entry-list))
  (message (format "Current entry set to %s" (car rascon-current-entry)))
  (setq rascon-current-passwd nil)
  rascon-current-entry)

;;
;; Dialup function.
(defun rascon-dialup (&optional nowait)
  "*Dialup following current dialup configuration."
  (interactive)
  (if (processp rascon-proc)
      (message "rascon is running. Try again later.")
    (if (interactive-p)
	(setq nowait t))
    (if (not rascon-current-entry)
	(rascon-set-current-entry))
    (setq rascon-dialup-complete nil)
    (setq rascon-saved-line "")
    (message "%s" (format "Dialing %s ... " (car rascon-current-entry)))
    (setq rascon-proc
	  (apply (function start-process)
		 "rascon"
		 " *RASCON*PROC*"
		 rascon-control-program
		 "-c" (nth 1 rascon-current-entry)
		 ;; "-c" (nth 1 rascon-current-entry) "-u" "-p"
		 (append
		  (if rascon-use-dialkeeper
		      (list "-k")
		    (and rascon-redial-num
			 rascon-redial-time
			 (integerp rascon-redial-num)
			 (integerp rascon-redial-time)
			 (list "-i"
			       (number-to-string rascon-redial-num)
			       (number-to-string rascon-redial-time)))))))
    (set-process-coding-system rascon-proc
			       file-name-coding-system
			       file-name-coding-system)
    (set-process-filter rascon-proc 'rascon-proc-filter)
    (set-process-sentinel rascon-proc 'rascon-proc-done)
    (process-kill-without-query (get-process rascon-proc))
    ;; Wait if called not interactive.
    (while (and (not nowait) rascon-proc)
      (sit-for 1)
      (discard-input))))

;;
;; Kill rascon process.
(defun rascon-kill ()
  (interactive)
  (if (processp rascon-proc)
      (if (or (not (interactive-p))
	      (y-or-n-p "Rascon process is running. KILL process? "))
	  (unwind-protect
	      (progn
		(interrupt-process rascon-proc)
		(kill-process rascon-proc)
		(if rascon-debug
		    (message "[RASCON-DEBUG] interrupt & kill process."))
		(rascon-hangup t))
	    (setq rascon-proc nil)))
    (message "Rascon process isn't running.")))

;;
;; Output filter for rascon process.
(defun rascon-proc-filter (proc output)
  (let ((case-fold-search nil))
    (setq rascon-saved-line (concat rascon-saved-line output))
    (cond
     ((string-match "Enter User name " rascon-saved-line)
      (setq rascon-saved-line "")
      (process-send-string
       proc
       (format "%s\n" (read-from-minibuffer
		       (format "[%s] Enter username : "
			       (car rascon-current-entry))))))
     ((string-match "Enter Password for " rascon-saved-line)
      (setq rascon-saved-line "")
      (process-send-string
       proc
       (format "%s\n"
	       (if rascon-current-passwd
		   rascon-current-passwd
		 (prog2
		     (require 'comint)
		     (setq rascon-current-passwd
			   (comint-read-noecho
			    (format "[%s] Enter password : "
				    (car rascon-current-entry)) t))
		   (message "")))))))
    (let (rascon-output-line)
      (while (string-match "\\([^\n]+\\)\n" rascon-saved-line)
	(setq rascon-output-line (substring rascon-saved-line
					    (match-beginning 1)
					    (match-end 1)))
	(setq rascon-saved-line (substring rascon-saved-line (match-end 0)))
	(if rascon-debug
	    (message "[RASCON-DEBUG] %s" rascon-output-line))
	(cond
	 ((string-match "^Success: " rascon-output-line)
	  (setq rascon-dialup-complete rascon-output-line))
	 ((string-match "^Error: " rascon-output-line)
	  (setq rascon-dialup-complete rascon-output-line))
	 ((string-match
	   "^Warning: Line is busy\\. Wait interval time and retry"
	   rascon-output-line)
	  (message "%s" rascon-output-line)
	  (sit-for 1)))))))

;;
;; Done process for rascon.
(defun rascon-proc-done (proc str)
  (if rascon-debug
      (message "[RASCON-DEBUG] process %s done has come (^^)V" proc))
  (setq rascon-proc nil)
  (if (not (stringp rascon-dialup-complete))
      (setq rascon-dialup-complete "something occur."))
  (message "%s" (format "Dialing %s ... %s"
			(car rascon-current-entry) rascon-dialup-complete))
  (sit-for 1)
  (cond
   ((string-match "^Success: " rascon-dialup-complete)
    (setq rascon-mode-indicator (concat "[ " (car rascon-current-entry) " ]"))
    (rascon-title-line-updater)
    (run-hooks 'rascon-dialup-complete-hook))
   ((string-match "^Error: " rascon-dialup-complete)
    (setq rascon-current-passwd nil)
    (run-hooks 'rascon-dialup-failure-hook))
   (t ;; seem rascon-kill
    (setq rascon-current-passwd nil)
    (run-hooks 'rascon-dialup-failure-hook)))
  (rascon-generate-entry-table)
  (setq rascon-dialup-complete nil))

;;
;; Hangup function.
;;
(defun rascon-hangup (&optional force)
  "*Hangup."
  (interactive "P")
  (if (and (processp rascon-proc) (not force))
      (message "rascon is running. Try again later.")
    (if (not (get-buffer rascon-buffer))
	(get-buffer-create rascon-buffer))
    (set-buffer rascon-buffer)
    (erase-buffer)
    (if (not force)
	(message
	 "%s"
	 (if rascon-current-entry
	     (format "Hangup %s ... " (car rascon-current-entry))
	   "Hangup ... ")))
    (apply (function call-process) rascon-control-program
	   nil rascon-buffer nil
	   (append
	    (if rascon-use-dialkeeper (list "-k"))
	    (list "-d")))
    (goto-char (point-min))
    (if (looking-at "Error:")
	(message "Hangup: %s"
		 (buffer-substring (point-min) (progn (end-of-line) (point))))
      (if rascon-current-entry
	  (setq rascon-current-entry (list (car rascon-current-entry)
					   (nth 1 rascon-current-entry)
					   "disconnect")))
      (rascon-generate-entry-table)
      (setq rascon-mode-indicator nil)
      (rascon-title-line-updater)
      (if (not force)
	  (message
	   "%s"
	   (if rascon-current-entry
	       (format "Hangup %s ... Done." (car rascon-current-entry))
	     "Hangup ... Done."))))))

;;;
;;;
(provide 'rascon)
;;; rascon.el ends here
